#include <ntddk.h>
#include <wmilib.h>
#include <initguid.h>
#include <ntintsafe.h>
#include <wdmsec.h>
#include <usbdrivr.h>
#include "driver.h"
#include "public.h"

#define NTSTRSAFE_LIB
#include <ntstrsafe.h>
#include <dontuse.h>

#ifndef BUSENUM_H
#define BUSENUM_H

#define BUSENUM_POOL_TAG (ULONG) 'VpcS'


#if DBG

#define DRIVERNAME "ScpFilter.sys: "

#define Bus_KdPrint(_x_) \
               DbgPrint (DRIVERNAME); \
               DbgPrint _x_;

#else

#define Bus_KdPrint(_x_)

#endif

extern ULONG BusEnumDebugLevel;

typedef enum _DEVICE_PNP_STATE {

    NotStarted = 0,         // Not started yet
    Started,                // Device has received the START_DEVICE IRP
    StopPending,            // Device has received the QUERY_STOP IRP
    Stopped,                // Device has received the STOP_DEVICE IRP
    RemovePending,          // Device has received the QUERY_REMOVE IRP
    SurpriseRemovePending,  // Device has received the SURPRISE_REMOVE IRP
    Deleted,                // Device has received the REMOVE_DEVICE IRP
    UnKnown                 // Unknown state

} DEVICE_PNP_STATE;

typedef struct _GLOBALS {

    UNICODE_STRING RegistryPath;

} GLOBALS;

extern GLOBALS Globals;

typedef struct _PENDING_IRP {

	LIST_ENTRY	Link;
	PIRP		Irp;

} PENDING_IRP, *PPENDING_IRP;

typedef struct _COMMON_DEVICE_DATA {

    PDEVICE_OBJECT  Self;

    BOOLEAN         IsFDO;

    DEVICE_PNP_STATE DevicePnPState;
    DEVICE_PNP_STATE PreviousPnPState;

    SYSTEM_POWER_STATE  SystemPowerState;
    DEVICE_POWER_STATE  DevicePowerState;

} COMMON_DEVICE_DATA, *PCOMMON_DEVICE_DATA;

typedef struct _PDO_DEVICE_DATA {

    #pragma warning(suppress:4201)
    COMMON_DEVICE_DATA;

    PDEVICE_OBJECT  ParentFdo;

    __drv_aliasesMem
    PWCHAR      HardwareIDs;

    ULONG SerialNo;

    LIST_ENTRY  Link;

    BOOLEAN     Present;
    BOOLEAN     ReportedMissing;
    UCHAR       Reserved[2]; // for 4 byte alignment

    ULONG       InterfaceRefCount;

    LIST_ENTRY  HoldingQueue;
    LIST_ENTRY  PendingQueue;
    KSPIN_LOCK  PendingQueueLock;

	UCHAR		Rumble[8];
	UCHAR		Report[20];

} PDO_DEVICE_DATA, *PPDO_DEVICE_DATA;

typedef struct _FDO_DEVICE_DATA {

    #pragma warning(suppress:4201)
    COMMON_DEVICE_DATA;

    PDEVICE_OBJECT  UnderlyingPDO;

    PDEVICE_OBJECT  NextLowerDriver;

    LIST_ENTRY      ListOfPDOs;

    ULONG           NumPDOs;

    FAST_MUTEX      Mutex;

    ULONG           OutstandingIO;

    KEVENT          RemoveEvent;
    KEVENT          StopEvent;

    UNICODE_STRING      InterfaceName;

} FDO_DEVICE_DATA, *PFDO_DEVICE_DATA;


#define FDO_FROM_PDO(pdoData) \
          ((PFDO_DEVICE_DATA) (pdoData)->ParentFdo->DeviceExtension)

#define INITIALIZE_PNP_STATE(_Data_)    \
        (_Data_)->DevicePnPState =  NotStarted;\
        (_Data_)->PreviousPnPState = NotStarted;

#define SET_NEW_PNP_STATE(_Data_, _state_) \
        (_Data_)->PreviousPnPState =  (_Data_)->DevicePnPState;\
        (_Data_)->DevicePnPState = (_state_);

#define RESTORE_PREVIOUS_PNP_STATE(_Data_)   \
        (_Data_)->DevicePnPState =   (_Data_)->PreviousPnPState;\

//
// Defined in BusEnum.c
//

DRIVER_INITIALIZE DriverEntry;

__drv_dispatchType(IRP_MJ_CREATE)
__drv_dispatchType(IRP_MJ_CLOSE)
DRIVER_DISPATCH Bus_CreateClose;

__drv_dispatchType(IRP_MJ_DEVICE_CONTROL)
DRIVER_DISPATCH Bus_IoCtl;

__drv_dispatchType(IRP_MJ_INTERNAL_DEVICE_CONTROL)
DRIVER_DISPATCH Bus_Internal_IoCtl;

DRIVER_UNLOAD Bus_DriverUnload;

VOID Bus_IncIoCount(__in PFDO_DEVICE_DATA Data);
VOID Bus_DecIoCount(__in PFDO_DEVICE_DATA Data);

NTSTATUS Bus_ReportDevice(PBUSENUM_REPORT_HARDWARE Report, PFDO_DEVICE_DATA fdoData, PUCHAR pBuffer);

VOID Bus_DrainPdo(__in PPDO_DEVICE_DATA pdoData);

//
// Defined in PNP.C
//

PCHAR PnPMinorFunctionString(UCHAR MinorFunction);

DRIVER_ADD_DEVICE Bus_AddDevice;

DRIVER_DISPATCH Bus_SendIrpSynchronously;

__drv_dispatchType(IRP_MJ_PNP)
DRIVER_DISPATCH Bus_PnP;

IO_COMPLETION_ROUTINE Bus_CompletionRoutine;

VOID Bus_InitializePdo(__drv_in(__drv_aliasesMem) PDEVICE_OBJECT Pdo, PFDO_DEVICE_DATA FdoData);

NTSTATUS Bus_PlugInDevice(PBUSENUM_PLUGIN_HARDWARE PlugIn, ULONG PlugInLength, PFDO_DEVICE_DATA DeviceData);

NTSTATUS Bus_UnPlugDevice(PBUSENUM_UNPLUG_HARDWARE UnPlug, PFDO_DEVICE_DATA DeviceData);

NTSTATUS Bus_EjectDevice(PBUSENUM_EJECT_HARDWARE Eject, PFDO_DEVICE_DATA FdoData);

void Bus_RemoveFdo(__in PFDO_DEVICE_DATA FdoData);

NTSTATUS Bus_DestroyPdo(PDEVICE_OBJECT Device, PPDO_DEVICE_DATA PdoData);

NTSTATUS Bus_FDO_PnP(__in PDEVICE_OBJECT DeviceObject, __in PIRP Irp, __in PIO_STACK_LOCATION IrpStack, __in PFDO_DEVICE_DATA DeviceData);

NTSTATUS Bus_StartFdo(__in PFDO_DEVICE_DATA FdoData, __in PIRP Irp);

PCHAR DbgDeviceIDString(BUS_QUERY_ID_TYPE Type);

PCHAR DbgDeviceRelationString(__in DEVICE_RELATION_TYPE Type);

//
// Defined in Power.c
//

NTSTATUS Bus_FDO_Power(PFDO_DEVICE_DATA FdoData, PIRP Irp);

NTSTATUS Bus_PDO_Power(PPDO_DEVICE_DATA PdoData, PIRP Irp);

__drv_dispatchType(IRP_MJ_POWER)
DRIVER_DISPATCH Bus_Power;

PCHAR PowerMinorFunctionString(UCHAR MinorFunction);

PCHAR DbgSystemPowerString(__in SYSTEM_POWER_STATE Type);

PCHAR DbgDevicePowerString(__in DEVICE_POWER_STATE Type);

//
// Defined in BusPDO.c
//

NTSTATUS Bus_PDO_PnP(__in PDEVICE_OBJECT DeviceObject, __in PIRP Irp, __in PIO_STACK_LOCATION IrpStack, __in PPDO_DEVICE_DATA DeviceData);

NTSTATUS Bus_PDO_QueryDeviceCaps(__in PPDO_DEVICE_DATA DeviceData, __in PIRP Irp);

NTSTATUS Bus_PDO_QueryDeviceId(__in PPDO_DEVICE_DATA DeviceData, __in PIRP Irp);

NTSTATUS Bus_PDO_QueryDeviceText(__in PPDO_DEVICE_DATA DeviceData, __in PIRP Irp);

NTSTATUS Bus_PDO_QueryResources(__in PPDO_DEVICE_DATA DeviceData, __in PIRP Irp);

NTSTATUS Bus_PDO_QueryResourceRequirements(__in PPDO_DEVICE_DATA DeviceData, __in PIRP Irp);

NTSTATUS Bus_PDO_QueryDeviceRelations(__in PPDO_DEVICE_DATA DeviceData, __in PIRP Irp);

NTSTATUS Bus_PDO_QueryBusInformation(__in PPDO_DEVICE_DATA DeviceData,__in PIRP Irp);

NTSTATUS Bus_GetDeviceCapabilities(__in PDEVICE_OBJECT DeviceObject, __out PDEVICE_CAPABILITIES DeviceCapabilities);

NTSTATUS Bus_PDO_QueryInterface(__in PPDO_DEVICE_DATA DeviceData, __in PIRP Irp);


VOID Bus_InterfaceReference(__in PVOID Context);

VOID Bus_InterfaceDereference(__in PVOID Context);

#endif
