﻿using System;
using System.ComponentModel;

namespace ScpControl 
{
    public partial class UsbMov : UsbDevice 
    {
        public static String MOV_USB_CLASS_GUID = "{2714024F-7B21-4D39-BD11-E6A4E8849E25}";

        protected new Byte[] m_Report = { 0x02, 0x00, 0x00, 0xFF, 0x00, 0x00, 0x00 };

        public UsbMov() : base(MOV_USB_CLASS_GUID) 
        {
            InitializeComponent();
        }

        public UsbMov(IContainer container) : base(MOV_USB_CLASS_GUID) 
        {
            container.Add(this);

            InitializeComponent();
        }


        public override Boolean Open(int Instance = 0)  
        {
            if (base.Open(Instance))
            {
                m_State = Ds3State.Reserved;

                Int32 Transfered = 0;

                if (SendTransfer(0xA1, 0x01, 0x0304, m_Buffer, ref Transfered))
                {
                    m_Master = new Byte[] { m_Buffer[15], m_Buffer[14], m_Buffer[13], m_Buffer[12], m_Buffer[11], m_Buffer[10] };
                    m_Local  = new Byte[] { m_Buffer[ 6], m_Buffer[ 5], m_Buffer[ 4], m_Buffer[ 3], m_Buffer[ 2], m_Buffer[ 1] };
                }
            }

            return State == Ds3State.Reserved;
        }

        public override Boolean Open(String DevicePath) 
        {
            if (base.Open(DevicePath))
            {
                m_State = Ds3State.Reserved;

                Int32 Transfered = 0;

                if (SendTransfer(0xA1, 0x01, 0x0304, m_Buffer, ref Transfered))
                {
                    m_Master = new Byte[] { m_Buffer[15], m_Buffer[14], m_Buffer[13], m_Buffer[12], m_Buffer[11], m_Buffer[10] };
                    m_Local  = new Byte[] { m_Buffer[ 6], m_Buffer[ 5], m_Buffer[ 4], m_Buffer[ 3], m_Buffer[ 2], m_Buffer[ 1] };
                }
            }

            return State == Ds3State.Reserved;
        }

        public override Boolean Start() 
        {
            if (IsActive)
            {
                Int32 Transfered = 0;

                m_State = Ds3State.Connected;
                m_Packet = 0;

                WriteIntPipe(m_Report, m_Report.Length, ref Transfered); LogDebug(String.Format("++ Move : Report [{0}]", Transfered));
            }

            return State == Ds3State.Connected;
        }


        public override Boolean Pair(Byte[] Master) 
        {
            Int32 Transfered = 0; Byte[] Buffer = { 0x05, Master[5], Master[4], Master[3], Master[2], Master[1], Master[0], 0x10, 0x01, 0x02, 0x12 };

            if (SendTransfer(0x21, 0x09, 0x0305, Buffer, ref Transfered))
            {
                for (Int32 Index = 0; Index < m_Master.Length; Index++)
                {
                    m_Master[Index] = Master[Index];
                }

                LogDebug(String.Format("++ Paired MOV [{0}] To BTH Dongle [{1}]", Local, Remote));
                return true;
            }

            return false;
        }
    }
}
